import es
import gamethread
import srcds.entities as ents

landmines = {}
hurters = {}

def round_start(ev):
    '''At the start of round clear our landmines info.'''
    landmines.clear()
    hurters.clear()

def upgrade_pack_used(ev):
    '''Finish deploying upgrades'''
    packid = int(ev['upgradeid'])
    userid = int(ev['userid'])
    pack = ents.get_entity(packid)
    # our cue!
    handle = es.createplayerlist(userid)[userid]['handle']
    landmines[packid] = Landmine(pack, handle)

def upgrade_pack_added(ev):
    '''Use upgrade'''
    packid = int(ev['upgradeid'])
    if packid in landmines:
        landmines[packid].weaken()


def player_hurt(ev):
    entindex = int(ev['attackerentid'] or '0')
    if entindex and entindex in hurters:
        mine = hurters[entindex]
        mine.explode()


def compare_distance(origin):
    def compare_to(landmine):
        return (origin-landmine.origin).length()
    return compare_to


def infected_hurt(ev):
    # for witch
    if ev['attacker'] == '0' and ev['amount'] == '10' and ev['type'] == '1024':
        # Chance!
        infected = ents.get_entity(int(ev['entityid']))
        origin = infected.origin
        mine = min(hurters.itervalues(), key=compare_distance(origin))
        mine.explode()


class Landmine(object):
    '''
    Class containing information and entities related to a mine.
    '''
    POWER_STEP = 1000
    POWER_STEPS = 4
    RADIUS = 30
    
    def __init__(self, pack, handle):
        '''
        Create a new mine around upgrade pack.
        '''
        self.packid = pack._index
        self.pack = pack
        self.origin = pack.origin
        # create explosion entity
        self.explosion = ents.env_explosion()
        self.explosion.iMagnitude = self.POWER_STEP * self.POWER_STEPS
        self.explosion.origin = self.origin + (0,0,64) # raise up a bit
        self.explosion.targetname = '%s_explosion' % self.packid
        self.explosion.spawnflags.set_flags('No Fireball', 'No Smoke', 'No Sparks', 'No particles')
        self.explosion.spawn()
        es.setindexprop(self.explosion._index, 'CBaseEntity.m_hOwnerEntity', handle)
        # create beam targets
        self.target1 = ents.prop_dynamic()
        self.target1.origin = self.origin + (self.RADIUS, 0, 3)
        self.target1.targetname = '%s_target1' % self.packid
        self.target1.model = 'models/w_models/weapons/w_eq_pipebomb.mdl'
        self.target1.rendermode = 'Dont Render'
        self.target1.spawn()
        self.target2 = ents.prop_dynamic()
        self.target2.origin = self.origin + (-self.RADIUS, 0, 3)
        self.target2.targetname = '%s_target2' % self.packid
        self.target2.model = 'models/w_models/weapons/w_eq_pipebomb.mdl'
        self.target2.rendermode = 'Dont Render'
        self.target2.spawn()
        # create beam
        self.beam = ents.env_beam()
        self.beam.spawnflags.set_flags('Ring')
        self.beam.life = 0.2
        self.beam.BoltWidth = 1.0
        self.beam.NoiseAmplitude = 0.0
        self.beam.texture = 'sprites/laserbeam.vmt'
        self.beam.StrikeTime = 1.0 # set to 0 when active
        self.beam.damage = 0.0
        self.beam.LightningStart = self.target1.targetname
        self.beam.LightningEnd = self.target2.targetname
        self.beam.TouchType = 0 # change to 4 when active
        self.beam.origin = self.origin
        self.beam.rendercolor = 0,255,0
        self.beam.renderamt = 200
        self.beam.spawn()
        # FIXME: how to detect explosion to be able to remove the mine?
        self.beam.TurnOn()
        # set it active in 5 seconds
        gamethread.delayed(5, self.activate)
    
    def activate(self):
        '''
        Activate the mine if possible...
        '''
        if self.packid in landmines:
            # okay, we are still here
            hurters[self.beam._index] = self
            self.beam.TouchType = 4
            self.beam.StrikeTime = 0.0
            self.beam.damage = 10.0
            self.update_beam_color()
    
    def update_beam_color(self):
        '''
        Check the power of the explosive and set color accordingly.
        '''
        power_level = int(self.explosion.iMagnitude / self.POWER_STEP)
        if power_level <= 1:
            self.beam.rendercolor = 255,0,0
        elif power_level == 2:
            self.beam.rendercolor = 255,192,0
        elif power_level == 3:
            self.beam.rendercolor = 128,0,255
        else:
            self.beam.rendercolor = 0,0,255
    
    def weaken(self):
        '''
        Reduce power.
        '''
        self.explosion.iMagnitude -= self.POWER_STEP
        if self.explosion.iMagnitude <= 0:
            self.remove()
        elif self.beam.TouchType == 4:
            self.update_beam_color()
    
    def explode(self):
        # L4D2 env_explosion has wrong sprites, looking ugly!
        # use prop_physics propanetank instead
        effect = ents.prop_physics()
        effect.origin = self.origin + (0,0,15)
        if self.pack.classname == 'upgrade_ammo_incendiary':
            effect.model = 'models/props_junk/gascan001a.mdl'
        else:
            effect.model = 'models/props_junk/propanecanister001a.mdl'
        effect.rendermode = 'Dont Render' # make it invisible :)
        effect.spawn()
        effect.Break()
        self.explosion.Explode()
        gamethread.delayed(0.1, self.remove)
    
    def remove(self):
        del landmines[self.packid]
        if self.beam._index in hurters:
            del hurters[self.beam._index]
        self.explosion.remove()
        self.target1.remove()
        self.target2.remove()
        self.beam.TurnOff()
        self.beam.remove()
        self.pack.remove()

